/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include <konkret/konkret.h>
#include "Cura_PowerManagementCapabilities.h"

#include "power.h"

static const CMPIBroker* _cb = NULL;

static void Cura_PowerManagementCapabilitiesInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = power_ref(_cb);
}

static void Cura_PowerManagementCapabilitiesInitializeMethod(CMPIMethodMI *mi)
{
    mi->hdl = power_ref(_cb);
}

static CMPIStatus Cura_PowerManagementCapabilitiesCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    power_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_PowerManagementCapabilitiesEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Cura_PowerManagementCapabilitiesEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    const char *ns = KNameSpace(cop);

    Cura_PowerManagementCapabilities w;
    Cura_PowerManagementCapabilities_Init(&w, _cb, ns);
    // TODO: make it unique
    Cura_PowerManagementCapabilities_Set_InstanceID(&w, "Linux:PowerManagementCapabilities");
    Cura_PowerManagementCapabilities_Set_ElementName(&w, "Linux:PowerManagementCapabilities");
    Cura_PowerManagementCapabilities_Set_Caption(&w, "Linux:PowerManagementCapabilities");

    int count;
    unsigned short *list = power_available_requested_power_states(mi->hdl, &count);
    Cura_PowerManagementCapabilities_Init_PowerStatesSupported(&w, count);
    for (int i = 0; i < count; i++) {
        Cura_PowerManagementCapabilities_Set_PowerStatesSupported(&w, i, list[i]);
    }

    // TODO: get this list dynamically from PowerStatesSupported (see SMASH)
    Cura_PowerManagementCapabilities_Init_PowerChangeCapabilities(&w, 3);
    Cura_PowerManagementCapabilities_Set_PowerChangeCapabilities(&w, 0, Cura_PowerManagementCapabilities_PowerChangeCapabilities_Power_State_Settable);
    Cura_PowerManagementCapabilities_Set_PowerChangeCapabilities(&w, 1, Cura_PowerManagementCapabilities_PowerChangeCapabilities_Power_Cycling_Supported);
    Cura_PowerManagementCapabilities_Set_PowerChangeCapabilities(&w, 2, Cura_PowerManagementCapabilities_PowerChangeCapabilities_Graceful_Shutdown_Supported);
    KReturnInstance(cr, w);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_PowerManagementCapabilitiesGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Cura_PowerManagementCapabilitiesCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_PowerManagementCapabilitiesModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_PowerManagementCapabilitiesDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_PowerManagementCapabilitiesExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    Cura_PowerManagementCapabilities,
    Cura_PowerManagementCapabilities,
    _cb,
    Cura_PowerManagementCapabilitiesInitialize(&mi))

static CMPIStatus Cura_PowerManagementCapabilitiesMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    power_unref(mi->hdl);
    mi->hdl = NULL;

    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_PowerManagementCapabilitiesInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return Cura_PowerManagementCapabilities_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    Cura_PowerManagementCapabilities,
    Cura_PowerManagementCapabilities,
    _cb,
    Cura_PowerManagementCapabilitiesInitializeMethod(&mi))

KONKRET_REGISTRATION(
    "root/cimv2",
    "Cura_PowerManagementCapabilities",
    "Cura_PowerManagementCapabilities",
    "instance method")
